<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    xmlns:oupca="http://www.oup.com/oupca"
    
    version="3.0">
    
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Author:</xd:b> Mark Dunn</xd:p>
            <xd:p><xd:b>Purpose:</xd:b> Generates list of relevant bibliographic references for sending to APIs to try and get IDs and DOIs.</xd:p>
            <xd:p>TO DO: filter references, e.g. ignore certain classes, bibItems without certain attributes, etc.</xd:p>
        </xd:desc>
    </xd:doc>
    
    
    
    <xd:doc scope="component">
        <xd:desc>Create WIP XML document containing only items representing bib references.</xd:desc>
    </xd:doc>
    <xsl:template match="/">
        <references>
            <xsl:apply-templates select="descendant::bibItem[@class = ('journalArticle', 'book', 'bookChapter')], descendant::*:mixed-citation[parent::*:ref][@publication-type = ('book', 'journal')]"/>
        </references>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Retain selected attributes of bibItem only.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="bibItem">
        <bibItem>
            <xsl:copy-of select="@id,@class,@date,@title,@journalName,@vol,@journalIssue,@page,@pubmedid,@webofknowledgeid,@doiTarget"/>
            <xsl:apply-templates select="@author"/>
            <xsl:apply-templates select="@journalName" mode="abbrev"/>
        </bibItem>
    </xsl:template>
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Parse bibItem @author attribute</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="@author">
        <xsl:variable name="names" as="xs:string+" select="tokenize(.,'\|')"/>
        <xsl:variable name="first-author" as="xs:string" select="$names[1]"/>
        
        <!-- assume format "Surname, Forenames" (but check this) -->
        <xsl:variable name="first-author-surname" as="xs:string" select="tokenize($first-author,',')[1]"/>
        <xsl:attribute name="author" select="string-join($names,', ')"/>
        <xsl:attribute name="firstAuthor">
            <xsl:choose>
                <xsl:when test="matches($first-author-surname,'[a-z]')">
                    <xsl:value-of select="$first-author-surname"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$first-author"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Parse components of BITS mixed-citation into bibItem attributes.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:mixed-citation" mode="bibItem">
        <bibItem>
            <xsl:apply-templates select="
                @id,
                parent::*:ref/@id,
                @publication-type,
                *:year,
                *:person-group[@person-group-type = 'author'][1],
                *:article-title,
                *:source,
                *:volume,
                *:issue,
                *:fpage,
                *:pub-id
                " mode="bibItem"/>
            <xsl:if test="@publication-type = 'journal'">
                <xsl:apply-templates select="*:source" mode="abbrev"/>    
            </xsl:if>
            <xsl:call-template name="getAuthors">
                <xsl:with-param name="names" select="*:person-group[@person-group-type = 'author']/*:string-name"/>
            </xsl:call-template>
        </bibItem>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>Get abbreviated journal title (BITS)</xd:desc>
    </xd:doc>
    <xsl:template match="*:source" mode="abbrev">
        <xsl:variable name="abbrev-title" as="xs:string" select="oupca:abbreviateTitle( string(.) )"/>
        <xsl:attribute name="journalAbbrevName" select="$abbrev-title"/>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>Get abbreviated journal title (OxChapML)</xd:desc>
    </xd:doc>
    <xsl:template match="@journalName" mode="abbrev">
        <xsl:variable name="abbrev-title" as="xs:string" select="oupca:abbreviateTitle( string(.) )"/>
        <xsl:attribute name="journalAbbrevName" select="$abbrev-title"/>
    </xsl:template>
    

    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/@publication-type to bibItem/@class</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="@publication-type" mode="bibItem">
        <xsl:attribute name="class" select="if (. = 'journal') then 'journalArticle' else ."/>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map @id to bibItem/@id</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="@id" mode="bibItem">
        <xsl:copy-of select="."/>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map ref/@id to bibItem/@id, if we have not already mapped the @id of the mixed-citation</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:ref/@id" mode="bibItem">
        <xsl:choose>
            <xsl:when test="child::*:mixed-citation/@id"/>
            <xsl:otherwise>
                <xsl:copy-of select="."/>
            </xsl:otherwise>
        </xsl:choose> 
    </xsl:template>


    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map BITS author to bibItem/@firstAuthor. Try to get surname only</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:person-group[@person-group-type = 'author']" mode="bibItem">
<!--        <xsl:attribute name="firstAuthor" select="oupca:getStringName(child::*:string-name[1])"/>-->
        
        <xsl:variable name="string-name-element" as="element()?" select="child::*:string-name[1]"/>
        
        <xsl:choose>
            <xsl:when test="exists($string-name-element)">
                <xsl:variable name="string-name-string" as="xs:string" select="string($string-name-element)"/>
                <xsl:attribute name="firstAuthor">
                    <xsl:choose>
                        <xsl:when test="$string-name-element/*:surname">
                            <xsl:value-of select="$string-name-element/*:surname[1]"/>
                        </xsl:when>
                        <xsl:when test="contains($string-name-string,',')">
                            <xsl:value-of select="tokenize($string-name-string,',')[1]"/>
                        </xsl:when>
                        <xsl:when test="matches(
                            tokenize($string-name-string,' ')[last()],
                            '[a-z]'
                            )">
                            <xsl:value-of select="tokenize($string-name-string,' ')[last()]"/>
                        </xsl:when>
                        <xsl:when test="matches(
                            tokenize($string-name-string,' ')[1],
                            '[a-z]'
                            )">
                            <xsl:value-of select="tokenize($string-name-string,' ')[1]"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="oupca:getStringName($string-name-element)"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:attribute>
            </xsl:when>
        </xsl:choose>
        
        
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>Get author names and concatenate into a string.</xd:desc>
        <xd:param name="names">Set of string-name elements</xd:param>
    </xd:doc>
    <xsl:template name="getAuthors">
        <xsl:param name="names" as="element()*"/>
        
        <xsl:if test="exists($names)">
            <xsl:variable name="names-as-string" as="xs:string*">
                <xsl:for-each select="$names">
                    <xsl:sequence select="oupca:getStringName(.)"/>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:attribute name="author" select="string-join($names-as-string, ', ')"/>
        </xsl:if>
        
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/year to bibItem/@date</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:year" mode="bibItem">
        <xsl:attribute name="date" select="./text()"/>
    </xsl:template>
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/volume to bibItem/@vol</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:volume" mode="bibItem">
        <xsl:attribute name="vol" select="./text()"/>
    </xsl:template>

    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/issue to bibItem/@journalIssue</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:issue" mode="bibItem">
        <xsl:attribute name="journalIssue" select="./text()"/>
    </xsl:template>
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/fpage to bibItem/@page</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:fpage" mode="bibItem">
        <xsl:attribute name="page" select="./text()"/>
    </xsl:template>


    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/article-title to bibItem/@title</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:article-title" mode="bibItem">
        <xsl:attribute name="title" select="oupca:truncateTitle( string(.) )"/>
        <xsl:attribute name="fullTitle" select="string(.)"/>
    </xsl:template>


    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/source to bibItem/@title or @journalName</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:source" mode="bibItem">
        <xsl:choose>
            <xsl:when test="../@publication-type = 'journal'">
                <xsl:attribute name="journalName" select="oupca:parseQueryTerm( string(.) )"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:attribute name="title" select="string(.)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Map mixed-citation/pub-id to bibItem/@pubmedid, @webofknowledgeid, @doiTarget</xd:p>
            <xd:p>Mapping described in file:/C:/SVN/software/Atlas/OUP2BITS/silverchair/OnlineProductsOUP/modules/oxchapml/_variables.xslt</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:pub-id" mode="bibItem">
        <xsl:choose>
            <xsl:when test="../@pub-id-type = 'doi'">
                <xsl:attribute name="doiTarget" select="string(.)"/>
            </xsl:when>
            <xsl:when test="../@pub-id-type = 'pmid'">
                <xsl:attribute name="pubmedid" select="string(.)"/>
            </xsl:when>
            <xsl:when test="../@pub-id-type = 'wos'">
                <xsl:attribute name="webofknowledgeid" select="string(.)"/>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:template>
    

    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Function oupca:getStringName() takes a string-name and returns a string consisting of surname, forenames,</xd:p>
        </xd:desc>
        <xd:param name="person-name">A string-name element identifying an author etc.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:getStringName" as="xs:string">
        <xsl:param name="person-name" as="element()"/>
        
        <!-- 
            Turn e.g. ""Bliss, T. V." into "Bliss TV"
            for better accuracy of PubMed searches
        -->
        <xsl:variable name="person-name-parsed" as="xs:string" select="
            replace(
                string($person-name),
                '(,|\.[\- ]?)',
                ''
            )
            "/>
         
        <xsl:variable name="surname-parsed" as="xs:string?" select="
            if (exists($person-name/*:surname/text()))
            then replace(
            string($person-name/*:surname/text()),
            ',',
            ''
            ) 
            else ()
            "/>
        
        <xsl:variable name="given-names-parsed" as="xs:string?" select="
            if (exists($person-name/*:given-names/text()))
            then replace(
                string($person-name/*:given-names/text()),
                '\. ?',
                ''
            ) 
            else ()
            "/>
        
        <xsl:variable name="name-components" as="xs:string*" select="$surname-parsed,$given-names-parsed"/>
        <xsl:choose>
            <xsl:when test="count($name-components) = 0">
                <xsl:value-of select="$person-name-parsed"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="string-join($name-components, ' ')"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Function oupca:parseQueryTerm() takes a string and removes characters that may cause a problem for the search APIs (e.g. terminal comma in journal name)</xd:p>
        </xd:desc>
        <xd:param name="query-term">A string for sending to an API.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:parseQueryTerm" as="xs:string">
        <xsl:param name="query-term" as="xs:string"/>
        
        <xsl:variable name="query-term-parsed-1" as="xs:string" select="replace($query-term,'(,|\.)$','')"/>
        
        <xsl:variable name="query-term-parsed-2" as="xs:string" select="replace($query-term-parsed-1,'^The ','')"/>
        
        
        <xsl:sequence select="$query-term-parsed-2"/>
        
    </xsl:function>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Function oupca:abbreviateTitle() takes a string and applies certain abbreviation rules</xd:p>
        </xd:desc>
        <xd:param name="title">A string representing a journal title.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:abbreviateTitle" as="xs:string">
        <xsl:param name="title" as="xs:string"/>
        
        <xsl:variable name="title-parsed-1" as="xs:string" select="replace($title,'(,|\.)$','')"/>
        
        <xsl:variable name="title-parsed-2" as="xs:string" select="replace($title-parsed-1,'^([A-Z][^ ]+)ol(ogy|ogical|ogist) ','$1ol ')"/>
        
        <xsl:variable name="title-parsed-3" as="xs:string" select="replace($title-parsed-2,' ([A-Z][^ ]+)ol(ogy|ogical|ogist)',' $1ol')"/>
        
        
        <xsl:variable name="title-parsed-4" as="xs:string" select="replace($title-parsed-3,' ([A-Z][^ ]+)ogr(aphy|aphic(al|))',' $1ogr')"/>
        
        <xsl:variable name="title-parsed-5" as="xs:string" select="replace($title-parsed-4,' ([A-Z][^ ]+)osc(opy|opic(al|))',' $1osc')"/>
        
        <!-- 
            Extras ...
            
            British (but not Brain) -> Br 
        
        -->
        <xsl:variable name="title-parsed-6" as="xs:string" select="replace($title-parsed-5,'(Abnorm|Acad|Addict|Adolesc|Adv|Affect|Afr|Am|Anat|Anim|Appl|Aquat|Arch|Assist|Assoc|Aviat|[Bb]ehav|Belg|Biomech|Biomed|Bull|Bus|[Cc]hem|[Cc]hir|Clin|Cogn|Comput|Conf|Contemp|Contrib|Crim|Crit|Curr|Def|Dent|Dev|Diagn|Disord|Econ|Educ|Elem|Emerg|Environ|Eur|Evid|Exp|Fam|Geriatr|Hist|Hosp|Hum|Hung|Ital|Lab|Lang|Learn|Lett|Mem|Metab|Med|Microb|Modif|Mol|Negl|Nucl|Nurs|Nutr|Obstet|Occup|Opin|Pa?ediatr|Palliat|press|Prev|Pract|Probl|Proc|Prog|Psychoanal|Psychosom|Regul|Reg|Relig|Rep|Res|Rev|Scand|[Ss]ci|Semin|Serv|Soc|Stat|Struct|Stud|Suppl|[Ss]urg|Symp|[Tt]heor|[Tt]her|Univ|[Vv]asc|Vet)[a-z]*','$1')"/>
        
        <xsl:variable name="title-parsed-7" as="xs:string" select="replace($title-parsed-6,'^The ','')"/>
        
        <xsl:variable name="title-parsed-8" as="xs:string" select="replace($title-parsed-7,'New (Engl|Y|Z)(and|ork|ealand)','N $1')"/>
        
      
        <xsl:variable name="title-parsed-9" as="xs:string" select="replace($title-parsed-8,'( &amp; | and | for (the |)| in (the |)| of (the |))',' ')"/>
        
        
        <xsl:variable name="title-words" as="xs:string+" select="tokenize($title-parsed-9,' ')"/>
        <xsl:variable name="title-words-abbreviated" as="xs:string+">
            <xsl:for-each select="$title-words">
                <xsl:sequence select="oupca:abbreviateSpecificWords(.)"/>
            </xsl:for-each>
        </xsl:variable>
        
        <xsl:variable name="title-parsed-10" as="xs:string" select="string-join($title-words-abbreviated,' ')"/>
        
        <xsl:variable name="title-parsed-11" as="xs:string" select="oupca:mapSpecificTitles($title-parsed-10)"/>
        
       
        <xsl:sequence select="$title-parsed-11"/>
        
    </xsl:function>
    

    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Function oupca:truncateTitle() takes a string and removes characters after a colon.</xd:p>
        </xd:desc>
        <xd:param name="title">A string for sending to an API.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:truncateTitle" as="xs:string">
        <xsl:param name="title" as="xs:string"/>
        
        <xsl:variable name="title-truncated-1" as="xs:string" select="replace($title,'^(.{30,}): .*$','$1')"/>
        <xsl:variable name="title-truncated-2" as="xs:string" select="replace($title-truncated-1,'^(.{30,}) in .*$','$1')"/>
        
        
        <xsl:sequence select="$title-truncated-2"/>
        
    </xsl:function>
    
    <xd:doc scope="component">
        <xd:desc>Function oupca:mapSpecificTitles() takes a string abbreviation and expands it to the full word, for certain specific journals (Nature, Science, etc.)</xd:desc>
        <xd:param name="title-abbrev">A string representing an abbreviated title.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:mapSpecificTitles" as="xs:string">
        <xsl:param name="title-abbrev" as="xs:string"/>
        
        <xsl:variable name="title-map" as="map(xs:string, xs:string)">
            <xsl:map>
                <xsl:map-entry key="'Nat'" select="'Nature'"/>
                <xsl:map-entry key="'Neurosci'" select="'Neuroscience'"/>
                <xsl:map-entry key="'Sci'" select="'Science'"/>
                <xsl:map-entry key="'Pediatr'" select="'Pediatrics'"/>
                <xsl:map-entry key="'Proc Nat Acad Sci'" select="'Proc Natl Acad Sci U S A'"/>
            </xsl:map>
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="exists(map:get($title-map,$title-abbrev))">
                <xsl:sequence select="map:get($title-map,$title-abbrev)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="$title-abbrev"/>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    
    
    <xd:doc scope="component">
        <xd:desc>Function oupca:abbreviateSpecificWords() takes a word from a journal title and returns an abbreviation if it matches certain patterns.</xd:desc>
        <xd:param name="title-word">A word from a journal title.</xd:param>
    </xd:doc>
    <xsl:function name="oupca:abbreviateSpecificWords" as="xs:string">
        <xsl:param name="title-word" as="xs:string"/>
        
        <xsl:variable name="abbrev-map" as="map(xs:string, xs:string)">
            <xsl:map>
                <xsl:map-entry key="'Affairs'" select="'Aff'"/>
                <xsl:map-entry key="'Annals'" select="'Ann'"/>
                <xsl:map-entry key="'Annual'" select="'Annu'"/>
                <xsl:map-entry key="'Attention'" select="'Atten'"/>
                <xsl:map-entry key="'Australian?'" select="'Aust'"/>
                <xsl:map-entry key="'Assessment'" select="'Assess'"/>
                <xsl:map-entry key="'Brit.*'" select="'Br'"/>
                <xsl:map-entry key="'Communic.*'" select="'Commun'"/>
                <xsl:map-entry key="'Comparative'" select="'Comp'"/>
                <xsl:map-entry key="'Consulting'" select="'Consult'"/>
                <xsl:map-entry key="'Diseases?'" select="'Dis'"/>
                <xsl:map-entry key="'Engineering'" select="'Eng'"/>
                <xsl:map-entry key="'Frontiers'" select="'Front'"/>
                <xsl:map-entry key="'General'" select="'Gen'"/>
                <xsl:map-entry key="'Genet.*'" select="'Genet'"/>
                <xsl:map-entry key="'Hormones?'" select="'Horm'"/>
                <xsl:map-entry key="'International'" select="'Int'"/>
                <xsl:map-entry key="'Interperson.*'" select="'Interpers'"/>
                <xsl:map-entry key="'J(ournal|rnl\.?).*'" select="'J'"/>
                <xsl:map-entry key="'Mechanic(al|s)'" select="'Mech'"/>
                <xsl:map-entry key="'Mechanisms?'" select="'Mech'"/>
                <xsl:map-entry key="'Mental'" select="'Ment'"/>
                <xsl:map-entry key="'Monographs?'" select="'Monogr'"/>
                <xsl:map-entry key="'National'" select="'Nat'"/>
                <xsl:map-entry key="'Natur.*'" select="'Nat'"/>
                <xsl:map-entry key="'Nervous'" select="'Nerv'"/>
                <xsl:map-entry key="'Person.*'" select="'Pers'"/>
                <xsl:map-entry key="'Psychiatric.*'" select="'Psychiatr'"/>
                <xsl:map-entry key="'Protocols?'" select="'Protoc'"/>
                <xsl:map-entry key="'Quarterly'" select="'Q'"/>
                <xsl:map-entry key="'Royal'" select="'R'"/>
                <xsl:map-entry key="'Systematic'" select="'Syst'"/>
                <xsl:map-entry key="'Traumatic'" select="'Trauma'"/>
            </xsl:map>
        </xsl:variable>
        
        <xsl:variable name="abbreviation" as="xs:string?">
            <xsl:for-each select="map:keys($abbrev-map)">
                <xsl:if test="matches($title-word,concat('^',.,'$'))">
                    <xsl:sequence select="map:get($abbrev-map,.)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="exists($abbreviation)">
                <xsl:sequence select="$abbreviation"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="$title-word"/>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
</xsl:stylesheet>