<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    
    xmlns:ixsl="http://saxonica.com/ns/interactiveXSLT"
    xmlns:js="http://saxonica.com/ns/globalJS"
    
    xmlns:cr="http://www.crossref.org/qschema/2.0"
    
    xmlns:oupca="http://www.oup.com/oupca"
    
    version="3.0">
    
    <xsl:import href="../../../lib/xsltModules/matchedTemplates/identity.xsl"/>
    <xsl:import href="filterReferences.xsl"/>
    
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Author:</xd:b> Mark Dunn</xd:p>
            <xd:p><xd:b>Purpose:</xd:b> Sends HTTP requsets to APIs containing information about bibliographic references. The HTTP response contains an identifier for the item (e.g. a DOI).</xd:p>
            
        </xd:desc>
    </xd:doc>
    
    <xsl:param name="report-target" as="xs:string" required="no" select="'report'"/>
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Root template</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="/">
        <xsl:message>[root] START</xsl:message>
        
        <xsl:apply-templates/>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Handle BITS mixed-citation</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="*:mixed-citation">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="child::node() except *:pub-id[@pub-id-type = ('doi','pmid','wos')]"/>
            
            
            <xsl:variable name="bibItem" as="element(bibItem)">
                <xsl:apply-templates select="." mode="bibItem"/>
            </xsl:variable>
            
            <xsl:result-document href="#{$report-target}" method="ixsl:append-content">
                <p>Handling reference with ID: <xsl:value-of select="parent::*:ref/@id"/></p>
                <p>Filtered reference: <xsl:sequence select="serialize($bibItem)"/></p>
            </xsl:result-document>
            
            <xsl:call-template name="crossref-query">
                <xsl:with-param name="citation" select="$bibItem"/>
            </xsl:call-template>
        </xsl:copy>
    </xsl:template>
    
    
    <xd:doc scope="component">
        <xd:desc>
            <xd:p>Create CrossRef query XML</xd:p>
            <xd:p><xd:a href="https://support.crossref.org/hc/en-us/articles/214844903-XML-query-examples">Examples</xd:a></xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template name="crossref-query">
        <xsl:param name="citation" as="element()" required="yes"/>
        
        <xsl:variable name="query" as="element()">
            <cr:query_batch version="2.0">
                <cr:head>
                    <cr:email_address>oup_data@oup.com</cr:email_address>
                    <cr:doi_batch_id>OUPReferencePoller</cr:doi_batch_id>
                </cr:head>
                <cr:body>
                    <cr:query key="{$citation/@id}" enabme-multiple-hits="true" forward-match="false">
                        <xsl:if test="$citation/@class = ('book','bookChapter') and exists($citation/@title)">
                            <cr:volume_title match="fuzzy">
                                <xsl:value-of select="$citation/@title"/>
                            </cr:volume_title>
                        </xsl:if>
                        
                        <xsl:if test="
                            $citation/@class = 'journalArticle' 
                            and exists($citation/@journalName)
                            and exists($citation/@fullTitle)
                            ">
                            <cr:journal_title match="fuzzy">
                                <xsl:value-of select="$citation/@journalName"/>
                            </cr:journal_title>
                            <cr:article_title match="fuzzy">
                                <xsl:value-of select="$citation/@fullTitle"/>
                            </cr:article_title>
                        </xsl:if>
                        
                        <xsl:if test="exists($citation/@firstAuthor)">
                            <cr:author match="{if (exists($citation/@page)) then 'optional' else 'fuzzy'}">
                                <xsl:value-of select="$citation/@firstAuthor"/>
                            </cr:author>
                        </xsl:if>
                        
                        <xsl:if test="exists($citation/@journalIssue)">
                            <cr:issue match="exact">
                                <xsl:value-of select="$citation/@journalIssue"/>
                            </cr:issue>
                        </xsl:if>
                        
                        <xsl:if test="exists($citation/@vol)">
                            <cr:volume>
                                <xsl:value-of select="$citation/@vol"/>
                            </cr:volume>
                        </xsl:if>
                        
                        <cr:first_page>
                            <xsl:choose>
                                <xsl:when test="exists($citation/@page)">
                                    <xsl:value-of select="$citation/@page"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="match" select="'null'"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </cr:first_page>
                        
                        <xsl:if test="exists($citation/@date)">
                            <cr:year>
                                <xsl:value-of select="$citation/@date"/>
                            </cr:year>
                        </xsl:if>
                    </cr:query>
                </cr:body>
                
            </cr:query_batch>
        </xsl:variable>
        
        <xsl:variable name="query-parameters" as="map(xs:string,xs:string)">
            <xsl:map>
                <xsl:map-entry key="'usr'" select="'oup-oso'"/>
                <xsl:map-entry key="'pwd'" select="'crpw1206'"/>
                <xsl:map-entry key="'format'" select="'xsd_xml'"/>
                <xsl:map-entry key="'qdata'" select="serialize($query)"/>
            </xsl:map>
        </xsl:variable>
        
        <xsl:variable name="query-string" as="xs:string">
            <xsl:variable name="items" as="xs:string+">
                <xsl:for-each select="map:keys($query-parameters)">
                    <xsl:sequence select="concat(., '=', map:get($query-parameters,.))"/>
                </xsl:for-each>
            </xsl:variable>
            <xsl:sequence select="string-join($items,'&amp;')"/>
        </xsl:variable>
        
        <xsl:variable name="http-submission" as="map(*)">
            <xsl:map>
                <xsl:map-entry key="'method'" select="'GET'"/>
                <xsl:map-entry key="'href'" select="
                    concat(
                    'https://doi.crossref.org/servlet/query',
                    '?',
                    $query-string
                    )
                    "/>
            </xsl:map>
        </xsl:variable>
        
        <xsl:result-document href="#{$report-target}" method="ixsl:append-content">
            <p>HTTP submission: <xsl:sequence select="serialize($http-submission)"/></p>
        </xsl:result-document>
        
        <ixsl:schedule-action http-request="$http-submission">
            <xsl:call-template name="crossref-handle-response"/>
        </ixsl:schedule-action>
        
    </xsl:template>
    
    
    <xsl:template name="crossref-handle-response">
        <xsl:context-item as="map(*)" use="required"/>
        <xsl:message>HTTP response status: <xsl:sequence select="serialize(?status)"/></xsl:message>
<!--        <xsl:message>HTTP response status message: <xsl:sequence select="serialize(?message)"/></xsl:message>-->
        <xsl:message>HTTP response headers: <xsl:sequence select="serialize(?headers)"/></xsl:message>
    </xsl:template>
    
</xsl:stylesheet>